@@ -23,13 +23,13 @@
 #include <kconfiggroup.h>
 #include <kdebug.h>
 #include <kdesktopfile.h>
-#include <kstandarddirs.h>
 #include <ksycoca.h>
 
 // Qt
 #include <QDir>
 #include <QStandardPaths>
 #include <QTest>
+#include <QSignalSpy>
 
 #include <mimetypedata.h>
 #include <mimetypewriter.h>
@@ -52,10 +52,11 @@ private Q_SLOTS:
         m_mimeTypeCreatedSuccessfully = false;
         QStringList appsDirs = QStandardPaths::standardLocations(QStandardPaths::ApplicationsLocation);
         //kDebug() << appsDirs;
-        m_localApps = appsDirs.first();
+        m_localApps = appsDirs.first() + '/';
+        m_localConfig = QStandardPaths::writableLocation(QStandardPaths::GenericConfigLocation);
         QVERIFY(QDir().mkpath(QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QStringLiteral("/mime/packages")));
 
-        QFile::remove(m_localApps + "mimeapps.list");
+        QFile::remove(m_localConfig + "mimeapps.list");
 
         // Create fake applications for some tests below.
         bool mustUpdateKSycoca = false;
@@ -74,7 +75,7 @@ private Q_SLOTS:
             mustUpdateKSycoca = true;
         }
 
-        QFile::remove(QStandardPaths::writableLocation(QStandardPaths::ConfigLocation) + QLatin1Char('/') + "filetypesrc");
+        QFile::remove(QStandardPaths::writableLocation(QStandardPaths::GenericConfigLocation) + QLatin1Char('/') + "filetypesrc");
 
         if ( mustUpdateKSycoca ) {
             // Update ksycoca in ~/.kde-unit-test after creating the above
@@ -136,6 +137,7 @@ private Q_SLOTS:
 
     void testMimeTypePatterns()
     {
+        // Given the text/plain mimetype
         QMimeDatabase db;
         MimeTypeData data(db.mimeTypeForName("text/plain"));
         QCOMPARE(data.name(), QString("text/plain"));
@@ -145,6 +147,8 @@ private Q_SLOTS:
         QStringList patterns = data.patterns();
         QVERIFY(patterns.contains("*.txt"));
         QVERIFY(!patterns.contains("*.toto"));
+
+        // When the user changes the patterns
         const QStringList origPatterns = patterns;
         patterns.removeAll("*.txt");
         patterns.append("*.toto"); // yes, a french guy wrote this, as you can see
@@ -155,23 +159,23 @@ private Q_SLOTS:
         bool needUpdateMimeDb = data.sync();
         QVERIFY(needUpdateMimeDb);
         MimeTypeWriter::runUpdateMimeDatabase();
-        //runKBuildSycoca();
+
+        // Then the GUI and the QMimeDatabase API should show the new patterns
         QCOMPARE(data.patterns(), patterns);
         data.refresh(); // reload from the xml
         QCOMPARE(data.patterns(), patterns);
-        // Check what's in ksycoca
+        // Check what's in QMimeDatabase
         QStringList newPatterns = db.mimeTypeForName("text/plain").globPatterns();
         newPatterns.sort();
         QCOMPARE(newPatterns, patterns);
         QVERIFY(!data.isDirty());
 
-        // Remove custom file
+        // And then removing the custom file by hand should revert to the initial state
         const QString packageFileName = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QLatin1String("/mime/") + "packages/text-plain.xml" ;
         QVERIFY(!packageFileName.isEmpty());
         QFile::remove(packageFileName);
         MimeTypeWriter::runUpdateMimeDatabase();
-        //runKBuildSycoca();
-        // Check what's in ksycoca
+        // Check what's in QMimeDatabase
         newPatterns = db.mimeTypeForName("text/plain").globPatterns();
         newPatterns.sort();
         QCOMPARE(newPatterns, origPatterns);
@@ -338,7 +342,7 @@ private: // helper methods
 
     void checkAddedAssociationsContains(const QString& mimeTypeName, const QString& application)
     {
-        const KConfig config(m_localApps + "mimeapps.list", KConfig::NoGlobals);
+        const KConfig config(m_localConfig + "mimeapps.list", KConfig::NoGlobals);
         const KConfigGroup group(&config, "Added Associations");
         const QStringList addedEntries = group.readXdgListEntry(mimeTypeName);
         if (!addedEntries.contains(application)) {
@@ -349,7 +353,7 @@ private: // helper methods
 
     void checkRemovedAssociationsContains(const QString& mimeTypeName, const QString& application)
     {
-        const KConfig config(m_localApps + "mimeapps.list", KConfig::NoGlobals);
+        const KConfig config(m_localConfig + "mimeapps.list", KConfig::NoGlobals);
         const KConfigGroup group(&config, "Removed Associations");
         const QStringList removedEntries = group.readXdgListEntry(mimeTypeName);
         if (!removedEntries.contains(application)) {
@@ -360,7 +364,7 @@ private: // helper methods
 
     void checkRemovedAssociationsDoesNotContain(const QString& mimeTypeName, const QString& application)
     {
-        const KConfig config(m_localApps + "mimeapps.list", KConfig::NoGlobals);
+        const KConfig config(m_localConfig + "mimeapps.list", KConfig::NoGlobals);
         const KConfigGroup group(&config, "Removed Associations");
         const QStringList removedEntries = group.readXdgListEntry(mimeTypeName);
         if (removedEntries.contains(application)) {
@@ -373,13 +377,19 @@ private: // helper methods
     {
         // Wait for notifyDatabaseChanged DBus signal
         // (The real KCM code simply does the refresh in a slot, asynchronously)
-        QEventLoop loop;
-        QObject::connect(KSycoca::self(), SIGNAL(databaseChanged(QStringList)), &loop, SLOT(quit()));
-        KProcess proc;
-        proc << KStandardDirs::findExe(KBUILDSYCOCA_EXENAME);
-        proc.setOutputChannelMode(KProcess::MergedChannels); // silence kbuildsycoca output
-        proc.execute();
-        loop.exec();
+
+        QProcess proc;
+        //proc.setProcessChannelMode(QProcess::ForwardedChannels);
+        const QString kbuildsycoca = QStandardPaths::findExecutable(KBUILDSYCOCA_EXENAME);
+        QVERIFY(!kbuildsycoca.isEmpty());
+        QStringList args;
+        args << "--testmode";
+        proc.start(kbuildsycoca, args);
+        QSignalSpy spy(KSycoca::self(), SIGNAL(databaseChanged(QStringList)));
+        proc.waitForFinished();
+        qDebug() << "waiting for signal";
+        QVERIFY(spy.wait(10000));
+        qDebug() << "got signal";
     }
 
     bool createDesktopFile(const QString& path)
@@ -408,6 +418,7 @@ private: // helper methods
     QString fakeApplication; // storage id of the fake application
     QString fakeApplication2; // storage id of the fake application2
     QString m_localApps;
+    QString m_localConfig;
     bool m_mimeTypeCreatedSuccessfully;
 };
 
